package com.agiledeveloper.archchecks;

import com.tngtech.archunit.core.domain.JavaClasses;
import com.tngtech.archunit.core.domain.JavaCodeUnit;
import com.tngtech.archunit.core.importer.ClassFileImporter;
import com.tngtech.archunit.junit.AnalyzeClasses;
import com.tngtech.archunit.lang.ArchCondition;
import com.tngtech.archunit.lang.ConditionEvents;
import com.tngtech.archunit.lang.syntax.ArchRuleDefinition;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.fail;

@AnalyzeClasses
public class CodeLevelChecks {
  private JavaClasses classes = new ClassFileImporter()
    .importPackages("com.agiledeveloper.processor");

  @Test
  void instanceofOnlyOnInterfaces(){
      ArchRuleDefinition.codeUnits()
        .should(new ArchCondition<JavaCodeUnit>("instanceof only to interfaces") {
          @Override
          public void check(JavaCodeUnit javaCodeUnit, ConditionEvents conditionEvents) {
            for(var instanceOfCheck : javaCodeUnit.getInstanceofChecks()) {
              var target = instanceOfCheck.getRawType();

              if(!target.isInterface()) {
                var message = String.format("instanceof usage in %s should check for an interface instead of a class",
                  instanceOfCheck.getSourceCodeLocation());
                fail(message);
              }
            }
          }
        })
        .check(classes);
  }

}
